lkml.org 
[lkml]   [2022]   [Nov]   [21]   [last100]   RSS Feed
Views: [wrap][no wrap]   [headers]  [forward] 
 
Messages in this thread
Patch in this message
/
From
Subject[patch V2 12/21] genirq/msi: Make descriptor allocation device domain aware
Date
Change the descriptor allocation and insertion functions to take a domain
id to prepare for the upcoming multi MSI domain per device support.

Signed-off-by: Thomas Gleixner <tglx@linutronix.de>
---
include/linux/msi.h | 16 +++++++++++++++-
kernel/irq/msi.c | 34 +++++++++++++++++++++++++---------
2 files changed, 40 insertions(+), 10 deletions(-)

--- a/include/linux/msi.h
+++ b/include/linux/msi.h
@@ -278,7 +278,21 @@ static inline void msi_desc_set_iommu_co
}
#endif

-int msi_insert_msi_desc(struct device *dev, struct msi_desc *init_desc);
+int msi_domain_insert_msi_desc(struct device *dev, unsigned int domid,
+ struct msi_desc *init_desc);
+/**
+ * msi_insert_msi_desc - Allocate and initialize a MSI descriptor in the
+ * default irqdomain and insert it at @init_desc->msi_index
+ * @dev: Pointer to the device for which the descriptor is allocated
+ * @init_desc: Pointer to an MSI descriptor to initialize the new descriptor
+ *
+ * Return: 0 on success or an appropriate failure code.
+ */
+static inline int msi_insert_msi_desc(struct device *dev, struct msi_desc *init_desc)
+{
+ return msi_domain_insert_msi_desc(dev, MSI_DEFAULT_DOMAIN, init_desc);
+}
+
void msi_free_msi_descs_range(struct device *dev, unsigned int first_index, unsigned int last_index);

/**
--- a/kernel/irq/msi.c
+++ b/kernel/irq/msi.c
@@ -63,7 +63,7 @@ static int msi_get_domain_base_index(str
* Return: pointer to allocated &msi_desc on success or %NULL on failure
*/
static struct msi_desc *msi_alloc_desc(struct device *dev, int nvec,
- const struct irq_affinity_desc *affinity)
+ const struct irq_affinity_desc *affinity)
{
struct msi_desc *desc = kzalloc(sizeof(*desc), GFP_KERNEL);

@@ -88,27 +88,42 @@ static void msi_free_desc(struct msi_des
kfree(desc);
}

-static int msi_insert_desc(struct msi_device_data *md, struct msi_desc *desc, unsigned int index)
+static int msi_insert_desc(struct device *dev, struct msi_desc *desc,
+ unsigned int domid, unsigned int index)
{
- int ret;
+ struct msi_device_data *md = dev->msi.data;
+ int baseidx, ret;
+
+ baseidx = msi_get_domain_base_index(dev, domid);
+ if (baseidx < 0) {
+ ret = baseidx;
+ goto fail;
+ }

desc->msi_index = index;
+ index += baseidx;
ret = xa_insert(&md->__store, index, desc, GFP_KERNEL);
if (ret)
- msi_free_desc(desc);
+ goto fail;
+ return 0;
+
+fail:
+ msi_free_desc(desc);
return ret;
}

/**
* msi_insert_msi_desc - Allocate and initialize a MSI descriptor and
* insert it at @init_desc->msi_index
*
* @dev: Pointer to the device for which the descriptor is allocated
+ * @domid: The id of the interrupt domain to which the desriptor is added
* @init_desc: Pointer to an MSI descriptor to initialize the new descriptor
*
* Return: 0 on success or an appropriate failure code.
*/
-int msi_insert_msi_desc(struct device *dev, struct msi_desc *init_desc)
+int msi_domain_insert_msi_desc(struct device *dev, unsigned int domid,
+ struct msi_desc *init_desc)
{
struct msi_desc *desc;

@@ -120,7 +135,8 @@ int msi_insert_msi_desc(struct device *d

/* Copy type specific data to the new descriptor. */
desc->pci = init_desc->pci;
- return msi_insert_desc(dev->msi.data, desc, init_desc->msi_index);
+
+ return msi_insert_desc(dev, desc, domid, init_desc->msi_index);
}

/**
@@ -143,7 +159,7 @@ static int msi_add_simple_msi_descs(stru
desc = msi_alloc_desc(dev, 1, NULL);
if (!desc)
goto fail_mem;
- ret = msi_insert_desc(dev->msi.data, desc, idx);
+ ret = msi_insert_desc(dev, desc, MSI_DEFAULT_DOMAIN, idx);
if (ret)
goto fail;
}
@@ -88,27 +88,42 @@ static void msi_free_desc(struct msi_des

/**
- * msi_insert_msi_desc - Allocate and initialize a MSI descriptor and
- * insert it at @init_desc->msi_index
+ * msi_domain_insert_msi_desc - Allocate and initialize a MSI descriptor and
+ * insert it at @init_desc->msi_index
*
* @dev: Pointer to the device for which the descriptor is allocated
\
 
 \ /
  Last update: 2022-11-21 15:40    [W:0.182 / U:0.484 seconds]
©2003-2020 Jasper Spaans|hosted at Digital Ocean and TransIP|Read the blog|Advertise on this site